@@ -245,7 +245,10 @@ static int offsetCmp(const void * avp, const void * bvp)
  */
 void headerUnsort(Header h)
 {
-    qsort(h->index, h->indexUsed, sizeof(*h->index), offsetCmp);
+    if (h->flags & HEADERFLAG_SORTED) {
+	qsort(h->index, h->indexUsed, sizeof(*h->index), offsetCmp);
+	h->flags &= ~HEADERFLAG_SORTED;
+    }
 }
 
 unsigned headerSizeof(Header h, int magicp)
@@ -384,6 +387,9 @@ static int regionSwab(indexEntry entry, int il, int dl,
 		const unsigned char * dataEnd,
 		int regionid)
 {
+    if ((entry != NULL && regionid >= 0) || (entry == NULL && regionid != 0))
+	return -1;
+
     for (; il > 0; il--, pe++) {
 	struct indexEntry_s ie;
 	rpm_tagtype_t type;
@@ -694,7 +700,6 @@ static void * doHeaderUnload(Header h,
     if (lengthPtr)
 	*lengthPtr = len;
 
-    h->flags &= ~HEADERFLAG_SORTED;
     headerSort(h);
 
     return (void *) ei;
@@ -840,7 +845,7 @@ Header headerLoad(void * uh)
 
 	{   int off = ntohl(pe->offset);
 
-	    if (hdrchkData(off))
+	    if (hdrchkData(off) || hdrchkRange(dl, off))
 		goto errxit;
 	    if (off) {
 		size_t nb = REGION_TAG_COUNT;
