@@ -165,7 +165,7 @@ static dbiIndex rpmdbOpenIndex(rpmdb db, rpmDbiTagVal rpmtag, int flags)
 						uintId, uintCmp, NULL);
 	    }
 	    /* If primary got created, we can safely run without fsync */
-	    if ((dbiFlags(dbi) & DBI_CREATED) || db->cfg.db_no_fsync) {
+	    if ((!verifyonly && (dbiFlags(dbi) & DBI_CREATED)) || db->cfg.db_no_fsync) {
 		rpmlog(RPMLOG_DEBUG, "disabling fsync on database\n");
                 db->cfg.db_no_fsync = 1;
 		dbSetFSync(db->db_dbenv, 0);
@@ -509,6 +509,7 @@ int rpmdbCheckTerminate(int terminate)
     if (terminating) {
 	rpmdb db;
 	rpmdbMatchIterator mi;
+	rpmdbIndexIterator ii;
 
 	while ((mi = rpmmiRock) != NULL) {
 	    rpmmiRock = mi->mi_next;
@@ -516,6 +517,12 @@ int rpmdbCheckTerminate(int terminate)
 	    mi = rpmdbFreeIterator(mi);
 	}
 
+	while ((ii = rpmiiRock) != NULL) {
+	    rpmiiRock = ii->ii_next;
+	    ii->ii_next = NULL;
+	    ii = rpmdbIndexIteratorFree(ii);
+	}
+
 	while ((db = rpmdbRock) != NULL) {
 	    rpmdbRock = db->db_next;
 	    db->db_next = NULL;
@@ -654,12 +661,14 @@ int rpmdbClose(rpmdb db)
 
     db = _free(db);
 
+    if (rpmdbRock == NULL) {
+	(void) rpmsqEnable(-SIGHUP, NULL);
+	(void) rpmsqEnable(-SIGINT, NULL);
+	(void) rpmsqEnable(-SIGTERM, NULL);
+	(void) rpmsqEnable(-SIGQUIT, NULL);
+	(void) rpmsqEnable(-SIGPIPE, NULL);
+    }
 exit:
-    (void) rpmsqEnable(-SIGHUP,	NULL);
-    (void) rpmsqEnable(-SIGINT,	NULL);
-    (void) rpmsqEnable(-SIGTERM,NULL);
-    (void) rpmsqEnable(-SIGQUIT,NULL);
-    (void) rpmsqEnable(-SIGPIPE,NULL);
     return rc;
 }
 
@@ -721,11 +730,13 @@ static int openDatabase(const char * prefix,
     /* Try to ensure db home exists, error out if we cant even create */
     rc = rpmioMkpath(rpmdbHome(db), 0755, getuid(), getgid());
     if (rc == 0) {
-	(void) rpmsqEnable(SIGHUP, NULL);
-	(void) rpmsqEnable(SIGINT, NULL);
-	(void) rpmsqEnable(SIGTERM,NULL);
-	(void) rpmsqEnable(SIGQUIT,NULL);
-	(void) rpmsqEnable(SIGPIPE,NULL);
+	if (rpmdbRock == NULL) {
+	    (void) rpmsqEnable(SIGHUP, NULL);
+	    (void) rpmsqEnable(SIGINT, NULL);
+	    (void) rpmsqEnable(SIGTERM, NULL);
+	    (void) rpmsqEnable(SIGQUIT, NULL);
+	    (void) rpmsqEnable(SIGPIPE, NULL);
+	}
 
 	/* Just the primary Packages database opened here */
 	rc = (rpmdbOpenIndex(db, RPMDBI_PACKAGES, db->db_flags) != NULL) ? 0 : -2;
@@ -1272,8 +1283,7 @@ rpmdbMatchIterator rpmdbFreeIterator(rpmdbMatchIterator mi)
 
     mi->mi_set = dbiFreeIndexSet(mi->mi_set);
     mi->mi_keyp = _free(mi->mi_keyp);
-    /* XXX rpmdbUnlink() should be rpmdbFree() but no such call exists... */
-    mi->mi_db = rpmdbUnlink(mi->mi_db);
+    rpmdbClose(mi->mi_db);
     mi->mi_ts = rpmtsFree(mi->mi_ts);
 
     mi = _free(mi);
@@ -2275,7 +2285,7 @@ rpmdbIndexIterator rpmdbIndexIteratorFree(rpmdbIndexIterator ii)
         xx = dbiCclose(ii->ii_dbi, ii->ii_dbc, 0);
     ii->ii_dbc = NULL;
     ii->ii_dbi = NULL;
-    ii->ii_db = rpmdbUnlink(ii->ii_db);
+    rpmdbClose(ii->ii_db);
     ii->ii_set = dbiFreeIndexSet(ii->ii_set);
 
     ii = _free(ii);
