@@ -291,7 +291,6 @@ rpmte rpmteFree(rpmte te)
 	rpmfsFree(te->fs);
 	rpmpsFree(te->probs);
 	rpmteCleanDS(te);
-	rpmtsFree(te->ts);
 
 	argvFree(te->collections);
 	argvFree(te->lastInCollectionsAny);
@@ -308,7 +307,7 @@ rpmte rpmteNew(rpmts ts, Header h, rpmElementType type, fnpyKey key,
 	       rpmRelocation * relocs)
 {
     rpmte p = xcalloc(1, sizeof(*p));
-    p->ts = rpmtsLink(ts);
+    p->ts = ts;
     p->type = type;
     addTE(p, h, key, relocs);
     switch (type) {
@@ -905,15 +904,19 @@ int rpmteProcess(rpmte te, pkgGoal goal)
 	}
     }
 
-    rpmteRunAllCollections(te, PLUGINHOOK_COLL_PRE_REMOVE);
+    if (!scriptstage) {
+	rpmteRunAllCollections(te, PLUGINHOOK_COLL_PRE_REMOVE);
+    }
 
     if (rpmteOpen(te, reset_fi)) {
 	failed = rpmpsmRun(te->ts, te, goal);
 	rpmteClose(te, reset_fi);
     }
     
-    rpmteRunAllCollections(te, PLUGINHOOK_COLL_POST_ADD);
-    rpmteRunAllCollections(te, PLUGINHOOK_COLL_POST_ANY);
+    if (!scriptstage) {
+	rpmteRunAllCollections(te, PLUGINHOOK_COLL_POST_ADD);
+	rpmteRunAllCollections(te, PLUGINHOOK_COLL_POST_ANY);
+    }
 
     /* XXX should %pretrans failure fail the package install? */
     if (failed && !scriptstage) {
